// Author: A.Voss@FH-Aachen.de
//
// lambda expressions
// tags: lambda, function

#include <iostream>
#include <functional>
#include <vector>
#include <algorithm>
using namespace std;

#include "../../include/tools.h"

// a function pointer type, float->double
typedef double (*fptr)(float);
// a function matching fptr
double div3(float x) { return x/3.0; }

// recap usage of function ptrs
void recap_fct_ptrs()
{
    float x=5;
    
    tools_log(); cout << "div3(x) = " << div3(x) << endl;  
    
    // a variable holding a function pointer
    fptr f = div3;
    //double (*f)(float);
    //f = div3;
    tools_log(); cout << "f(x)    = " << f(x) << endl;  
    
    // a general type of function ptr, float->double
    function<double(float)> g = div3;
    tools_log(); cout << "g(x)    = " << g(x) << endl;
}

// numerical integration (quadrature)
double sehnentrapezregel(fptr f, float a, float b) { return 0.5*(f(b)+f(a))*(b-a); }

// function for integration
double x2(float x) { return x*x; }
double x3(float x) { return x*x*x; }

void num_integration()
{
    float a=1.0f, b=2.0f;
    
    double A;
    
    A = sehnentrapezregel(x2,a,b);
    tools_log(); cout << "integral(x^2)_[a,b] = " << A << endl;

    A = sehnentrapezregel(x3,a,b);
    tools_log(); cout << "integral(x^3)_[a,b] = " << A << endl;
    
    // this is a lambda expression
    A = sehnentrapezregel( [](float x) -> double {return (x*x);}, a, b);
    tools_log(); cout << "integral(x^2)_[a,b] = " << A << endl;

    // or this (another syntax)
    A = sehnentrapezregel( [](float x) {return (double)(x*x*x);}, a, b);
    tools_log(); cout << "integral(x^3)_[a,b] = " << A << endl;
    
    // variables holding a lambda expression
    auto f2 = [](float x) -> double {return (x*x);};
    A = sehnentrapezregel(f2,a,b);
    tools_log(); cout << "integral(x^2)_[a,b] = " << A << endl;

    auto f3 = [](float x) {return (double)(x*x*x);};
    A = sehnentrapezregel(f3,a,b);
    tools_log(); cout << "integral(x^3)_[a,b] = " << A << endl;
}

int main()
{
    tools_log(); cout << "recap function pointers" << endl;
    recap_fct_ptrs();
    
    cout << endl;

    tools_log(); cout << "example for function pointers" << endl;
    num_integration();
    
    return EXIT_SUCCESS;
}
